@@ -1,26 +1,29 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-
+###############################################################################
+# Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
+#                All rights reserved.                                         #
+# This file is part of the Masci-tools package.                               #
+# (Material science tools)                                                    #
+#                                                                             #
+# The code is hosted on GitHub at https://github.com/judftteam/masci-tools.   #
+# For further information on the license, see the LICENSE.txt file.           #
+# For further information please visit http://judft.de/.                      #
+#                                                                             #
+###############################################################################
 """
 Tools for the impurity caluclation plugin and its workflows
 """
-#use print('message') instead of print 'message' in python 2.7 as well:
-from __future__ import print_function
-# redefine raw_input for python 3/2.7 compatilbility
-from __future__ import absolute_import
-from builtins import object
 from sys import version_info
-from six.moves import range
-from six.moves import input
 if version_info[0] >= 3:
+
     def raw_input(msg):
         return eval(eval(input(msg)))
 
-__copyright__ = (u"Copyright (c), 2018, Forschungszentrum Jülich GmbH,"
-                 "IAS-1/PGI-1, Germany. All rights reserved.")
-__license__ = "MIT license, see LICENSE.txt file"
-__version__ = "0.3"
-__contributors__ = u"Philipp Rüßmann"
+
+__copyright__ = (u'Copyright (c), 2018, Forschungszentrum Jülich GmbH,' 'IAS-1/PGI-1, Germany. All rights reserved.')
+__license__ = 'MIT license, see LICENSE.txt file'
+__version__ = '0.3'
+__contributors__ = u'Philipp Rüßmann'
 
 
 class modify_potential(object):
@@ -29,13 +32,13 @@ class modify_potential(object):
     """
 
     def _check_potstart(self, str1, mode='pot', shape_ver='new'):
-        if mode=='shape':
-            if shape_ver=='new':
-                check1='Shape number' in str1
+        if mode == 'shape':
+            if shape_ver == 'new':
+                check1 = 'Shape number' in str1
             else:
-                check1= (len(str1)==11)
+                check1 = (len(str1) == 11)
         else:
-            check1='exc:' in str1
+            check1 = 'exc:' in str1
         return check1
 
     def _read_input(self, filepath):
@@ -51,22 +54,25 @@ class modify_potential(object):
         #print(mode, len(data))
 
         # read file
-        index1=[];index2=[]
+        index1 = []
+        index2 = []
         for i in range(len(data)):
-          if self._check_potstart(data[i], mode=mode):
-            index1.append(i)
-            if len(index1)>1: index2.append(i-1)
+            if self._check_potstart(data[i], mode=mode):
+                index1.append(i)
+                if len(index1) > 1:
+                    index2.append(i - 1)
         index2.append(i)
 
         # read shapefun if old style is used
-        if mode=='shape' and len(index1)<1:
-            index1=[];index2=[]
+        if mode == 'shape' and len(index1) < 1:
+            index1 = []
+            index2 = []
             for i in range(len(data)):
                 if self._check_potstart(data[i], mode=mode, shape_ver='old'):
                     index1.append(i)
-                if len(index1)>1: index2.append(i-1)
+                if len(index1) > 1:
+                    index2.append(i - 1)
                 index2.append(i)
-
         """
         print(index1)
         print(index2)
@@ -89,28 +95,31 @@ class modify_potential(object):
         """
         index1, index2, data = self._read_input(shapefun_path)
 
-        order=list(range(len(index1)))
+        order = list(range(len(index1)))
 
-        natomtemp = int(open(scoefpath).readlines()[0])
-        filedata=open(scoefpath).readlines()[1:natomtemp+1]
-        listnew=[]
+        with open(scoefpath, 'r') as f:
+            lines = f.readlines()
+            natomtemp = int(lines[0])
+            filedata = lines[1:natomtemp + 1]
+
+        listnew = []
         for line in filedata:
-            if (len(line.split())>1):
-                listnew.append(atom2shapes[int(line.split()[3])-1]-1)
+            if len(line.split()) > 1:
+                listnew.append(atom2shapes[int(line.split()[3]) - 1] - 1)
         order = listnew
 
-        datanew=[]
+        datanew = []
         for i in range(len(order)):
-          for ii in range(index1[order[i]], index2[order[i]]+1  ):
-            datanew.append(data[ii])
+            for ii in range(index1[order[i]], index2[order[i]] + 1):
+                datanew.append(data[ii])
 
         # add header to shapefun_new
         tmp = datanew
         datanew = []
-        datanew.append('   %i\n' %(len(order)))
+        datanew.append('   %i\n' % (len(order)))
         datanew.append('  1.000000000000E+00\n')
         datanew += tmp
-        with open(shapefun_new,'w') as f:
+        with open(shapefun_new, 'w') as f:
             f.writelines(datanew)
 
     def neworder_potential(self, potfile_in, potfile_out, neworder, potfile_2=None, replace_from_pot2=None):
@@ -148,7 +157,7 @@ class modify_potential(object):
                 raise ValueError('replace_from_pot2 not given')
             else:
                 replace_from_pot2 = array(replace_from_pot2)
-                if shape(replace_from_pot2)[1]!=2:
+                if shape(replace_from_pot2)[1] != 2:
                     raise ValueError('replace_from_pot2 needs to be a 2D array!')
         else:
             if replace_from_pot2 is not None:
@@ -158,17 +167,17 @@ class modify_potential(object):
         # ensure that numbers are integers:
         order = [int(i) for i in neworder]
 
-        datanew=[]
+        datanew = []
         for i in range(len(order)):
             # check if new position is replaced with position from old pot
-            if replace_from_pot2 is not None and i in replace_from_pot2[:,0]:
-                    replace_index = replace_from_pot2[replace_from_pot2[:,0]==i][0][1]
-                    for ii in range(index12[replace_index], index22[replace_index]+1 ):
-                        datanew.append(data2[ii])
-            else: # otherwise take new potntial according to input list
-                    for ii in range(index1[order[i]], index2[order[i]]+1 ):
-                        datanew.append(data[ii])
+            if replace_from_pot2 is not None and i in replace_from_pot2[:, 0]:
+                replace_index = replace_from_pot2[replace_from_pot2[:, 0] == i][0][1]
+                for ii in range(index12[replace_index], index22[replace_index] + 1):
+                    datanew.append(data2[ii])
+            else:  # otherwise take new potntial according to input list
+                for ii in range(index1[order[i]], index2[order[i]] + 1):
+                    datanew.append(data[ii])
 
         # write out new potential
-        with open(potfile_out,'w') as f:
+        with open(potfile_out, 'w') as f:
             f.writelines(datanew)
